@@ -25,7 +25,7 @@ package Spreadsheet::Read;
 use strict;
 use warnings;
 
-our $VERSION = "0.58";
+our $VERSION = "0.59";
 sub  Version { $VERSION }
 
 use Carp;
@@ -56,7 +56,8 @@ my %can = map { $_->[0] => 0 } @parsers;
 for (@parsers) {
     my ($flag, $mod, $vsn) = @$_;
     $can{$flag} and next;
-    eval "require $mod; $vsn and ${mod}->VERSION ($vsn); \$can{\$flag} = '$mod'";
+    eval "require $mod; qq{$vsn} and ${mod}->VERSION ($vsn); \$can{\$flag} = '$mod'";
+    #$@ && $@ !~ m{Can't locate} and warn "$mod: $@\n";
     }
 $can{sc} = __PACKAGE__;	# SquirelCalc is built-in
 
@@ -325,6 +326,7 @@ sub ReadData
 		maxcol	=> 0,
 		cell	=> [],
 		attr	=> [],
+		merged  => [],
 		},
 	    );
 
@@ -413,21 +415,24 @@ sub ReadData
 		     : ($io_fil && $txt =~ m/\.(xlsx?)$/i && ($_parser = $1))) {
 	my $parse_type = $_parser =~ m/x$/i ? "XLSX" : "XLS";
 	$can{lc $parse_type} or croak "Parser for $parse_type is not installed";
-	my $oBook;
 	$debug and print STDERR "Opening $parse_type \$txt\n";
-	if ($io_ref) {
-	    $oBook = $parse_type eq "XLSX"
+	my $oBook = eval {
+	    $io_ref
+	      ? $parse_type eq "XLSX"
 		? $can{xlsx} =~ m/::XLSX/
 		? Spreadsheet::XLSX->new ($io_ref)
 		: Spreadsheet::ParseXLSX->new (%parser_opts)->parse ($io_ref)
-		: Spreadsheet::ParseExcel->new (%parser_opts)->Parse ($io_ref);
-	    }
-	else {
-	    $oBook = $parse_type eq "XLSX"
+		: Spreadsheet::ParseExcel->new (%parser_opts)->Parse ($io_ref)
+	      : $parse_type eq "XLSX"
 		? $can{xlsx} =~ m/::XLSX/
 		? Spreadsheet::XLSX->new ($txt)
 		: Spreadsheet::ParseXLSX->new (%parser_opts)->parse ($txt)
 		: Spreadsheet::ParseExcel->new (%parser_opts)->Parse ($txt);
+	    };
+	unless ($oBook) {
+	    # cleanup will fail on folders with spaces.
+	    (my $msg = $@) =~ s/ at \S+ line \d+.*//s;
+	    croak "$parse_type parser cannot parse data: $msg";
 	    }
 	$debug > 8 and _dump (oBook => $oBook);
 	my @data = ( {
@@ -465,10 +470,17 @@ sub ReadData
 		maxcol	=> 0,
 		cell	=> [],
 		attr	=> [],
+		merged  => [],
 		);
 	    defined $sheet{label}  or  $sheet{label}  = "-- unlabeled --";
 	    exists $oWkS->{MaxRow} and $sheet{maxrow} = $oWkS->{MaxRow} + 1;
 	    exists $oWkS->{MaxCol} and $sheet{maxcol} = $oWkS->{MaxCol} + 1;
+	    $sheet{merged} = [
+		map  {  $_->[0] }
+		sort {  $a->[1] cmp $b->[1] }
+		map  {[ $_, pack "NNNN", @$_          ]}
+		map  {[ map { $_ + 1 } @{$_}[1,0,3,2] ]}
+		@{$oWkS->get_merged_areas || []}];
 	    my $sheet_idx = 1 + @data;
 	    $debug and print STDERR "\tSheet $sheet_idx '$sheet{label}' $sheet{maxrow} x $sheet{maxcol}\n";
 	    if (exists $oWkS->{MinRow}) {
@@ -546,9 +558,9 @@ sub ReadData
 
 				type    => lc $oWkC->{Type},
 				enc     => $oWkC->{Code},
-				merged  => $oWkC->{Merged} || 0,
-				hidden  => $FmT->{Hidden}  || 0,
-				locked  => $FmT->{Lock}    || 0,
+				merged  => $oWkC->is_merged || 0,
+				hidden  => $FmT->{Hidden}   || 0,
+				locked  => $FmT->{Lock}     || 0,
 				format  => $fmi,
 				halign  => [ undef, qw( left center right
 					   fill justify ), undef,
@@ -612,6 +624,7 @@ sub ReadData
 		maxcol	=> 0,
 		cell	=> [],
 		attr	=> [],
+		merged  => [],
 		},
 	    );
 
@@ -682,11 +695,12 @@ sub ReadData
 	    foreach my $sheet (@sheets) {
 		my @sheet = @{$sheet->{data}};
 		my %sheet = (
-		    label	=> $sheet->{label},
-		    maxrow	=> scalar @sheet,
-		    maxcol	=> 0,
-		    cell	=> [],
-		    attr	=> [],
+		    label  => $sheet->{label},
+		    maxrow => scalar @sheet,
+		    maxcol => 0,
+		    cell   => [],
+		    attr   => [],
+		    merged => [],
 		    );
 		my $sheet_idx = 1 + @data;
 		$debug and print STDERR "\tSheet $sheet_idx '$sheet{label}' $sheet{maxrow} rows\n";
@@ -758,6 +772,8 @@ The data is returned as an array reference:
           [ undef, 1 ],
           [ undef, undef, undef, undef, undef, "Nugget" ],
           ],
+        attr    => [],
+        merged  => [],
         A1      => 1,
         B5      => "Nugget",
         },
@@ -1033,6 +1049,7 @@ is made to analyze and store field attributes like this:
 	  locked  => 0,
 	  enc     => "iso8859-1",
 	  }, ]
+      merged => [],
       A1     => 1,
       B5     => "Nugget",
       },
@@ -1040,6 +1057,70 @@ is made to analyze and store field attributes like this:
 This has now been partially implemented, mainly for Excel, as the other
 parsers do not (yet) support all of that. YMMV.
 
+=head3 Merged cells
+
+Note that only [Spreadsheet::ReadSXC] documents the use of merged cells,
+and not in a way useful for the spreadsheet consumer.
+
+CSV does not support merged cells (though future implementations of CSV
+for the web might).
+
+The documentation of merged areas in [Spreadsheet::ParseExcel] and
+[Spreadsheet::ParseXLSX] can be found in [Spreadsheet::ParseExcel::Worksheet]
+and [Spreadsheet::ParseExcel::Cell].
+
+None of basic [Spreadsheet::XLSX], [Spreadsheet::ParseExcel], and
+[Spreadsheet::ParseXLSX] manual pages mention merged cells at all.
+
+This module just tries to return the information in a generic way.
+
+Given this spreadsheet as an example
+
+ merged.xlsx:
+ 
+     A     B     C
+  +-----+-----------+
+ 1|     | foo       |
+  +-----+           +
+ 2| bar |           |
+  |     +-----+-----+
+ 3|     | urg | orc |
+  +-----+-----+-----+
+
+the information extracted from that undocumented information is
+returned in the C<merged> entry of the sheet's hash as a list of
+top-left, bottom-right coordinate pars (col, row, col, row). For
+given example, that would be:
+
+ $ss->{merged} = [
+    [ 1, 2, 1, 3 ], # A2-A3
+    [ 2, 1, 3, 2 ], # B1-C2
+    ];
+
+When the attributes are also enabled, there is some merge information
+copied directly from the cell information, but again, that stems from
+code analysis and not from documentation:
+
+ my $ss = ReadData ("merged.xlsx", attr => 1)->[1];
+ foreach my $row (1 .. $ss->{maxrow}) {
+     foreach my $col (1 .. $ss->{maxcol}) {
+         my $cell = cr2cell ($col, $row);
+         printf "%s %-3s %d  ", $cell, $ss->{$cell},
+             $ss->{attr}[$col][$row]{merged};
+         }
+     print "\n";
+     }
+
+ A1     0  B1 foo 1  C1     1
+ A2 bar 1  B2     1  C2     1
+ A3     1  B3 urg 0  C3 orc 0
+
+In this example, there is no way to see if B2 is merged to A2 or
+to B1 without analyzing all surrounding cells. This could as well
+mean A2:A3, B1:C1, B2: C2, as A2:A3, B1:B2, C1:C2, as A2:A3, B1:C2.
+Use the C<merged> entry described above to find out what fields are
+merged to what other fields.
+
 =head1 TOOLS
 
 This modules comes with a few tools that perform tasks from the FAQ, like
